@@ -7,8 +7,8 @@ from collections import deque
 class StringChain(object):
     def __init__(self):
         self.d = deque()
-        self.ig = 0
-        self.tailig = 0
+        self.ignored = 0
+        self.tailignored = 0
         self.len = 0
 
     def append(self, s):
@@ -18,9 +18,9 @@ class StringChain(object):
             return
 
         # First trim off any ignored tail bytes.
-        if self.tailig:
-            self.d[-1] = self.d[-1][:-self.tailig]
-            self.tailig = 0
+        if self.tailignored:
+            self.d[-1] = self.d[-1][:-self.tailignored]
+            self.tailignored = 0
 
         self.d.append(s)
         self.len += len(s)
@@ -38,27 +38,27 @@ class StringChain(object):
         else:
             return ''
 
-    def popleft(self, l):
-        """ Remove the leading l bytes of the chain and return them as a new
-        StringChain object. """
+    def popleft_new_stringchain(self, bytes):
+        """ Remove some of the leading bytes of the chain and return them as a
+        new StringChain object. (Use str() on it if you want the bytes in a
+        string, or call popleft() instead of popleft_new_stringchain().) """
         assert self._assert_invariants()
-        if not l or not self.d:
+        if not bytes or not self.d:
             return self.__class__()
 
-        precondition(l >= 0, l)
+        precondition(bytes >= 0, bytes)
 
         # We need to add at least this many bytes to the new StringChain.
-        bytesleft = l + self.ig
+        bytesleft = bytes + self.ignored
         n = self.__class__()
-        n.ig = self.ig
+        n.ignored = self.ignored
 
         while bytesleft > 0 and self.d:
             s = self.d.popleft()
-            self.len -= (len(s) - self.ig)
+            self.len -= (len(s) - self.ignored)
             n.d.append(s)
-            # If adding this string will put more than enough bytes into n, set its .len to only the desired length.
-            n.len += len(s)-self.ig
-            self.ig = 0
+            n.len += (len(s)-self.ignored)
+            self.ignored = 0
             bytesleft -= len(s)
 
         overrun = - bytesleft
@@ -66,52 +66,96 @@ class StringChain(object):
         if overrun > 0:
             self.d.appendleft(s)
             self.len += overrun
-            self.ig = len(s) - overrun
+            self.ignored = len(s) - overrun
             n.len -= overrun
-            n.tailig = overrun
+            n.tailignored = overrun
         else:
-            self.ig = 0
+            self.ignored = 0
 
         # Either you got exactly how many you asked for, or you drained self entirely and you asked for more than you got.
-        assert (n.len == l) or ((not self.d) and (l > self.len)), (n.len, l, len(self.d))
+        assert (n.len == bytes) or ((not self.d) and (bytes > self.len)), (n.len, bytes, len(self.d))
 
         assert self._assert_invariants()
         assert n._assert_invariants()
         return n
 
+    def popleft(self, bytes):
+        """ Remove some of the leading bytes of the chain and return them as a
+        string. """
+        assert self._assert_invariants()
+        if not bytes or not self.d:
+            return ''
+
+        precondition(bytes >= 0, bytes)
+
+        # We need to add at least this many bytes to the result.
+        bytesleft = bytes
+        resstrs = []
+
+        s = self.d.popleft()
+        if self.ignored:
+            s = s[self.ignored:]
+            self.ignored = 0
+        self.len -= len(s)
+        resstrs.append(s)
+        bytesleft -= len(s)
+
+        while bytesleft > 0 and self.d:
+            s = self.d.popleft()
+            self.len -= len(s)
+            resstrs.append(s)
+            bytesleft -= len(s)
+
+        overrun = - bytesleft
+
+        if overrun > 0:
+            self.d.appendleft(s)
+            self.ignored = (len(s) - overrun)
+            self.len += overrun
+            resstrs[-1] = resstrs[-1][:-overrun]
+
+        resstr = ''.join(resstrs)
+
+        # Either you got exactly how many you asked for, or you drained self entirely and you asked for more than you got.
+        assert (len(resstr) == bytes) or ((not self.d) and (bytes > self.len)), (len(resstr), bytes, len(self.d), overrun)
+
+        assert self._assert_invariants()
+
+        return resstr
+
     def __len__(self):
         assert self._assert_invariants()
         return self.len
 
-    def trim(self, l):
-        """ Trim off the leading l bytes. """
+    def trim(self, bytes):
+        """ Trim off some of the leading bytes. """
         assert self._assert_invariants()
-        self.ig += l
-        self.len -= l
-        while self.d and self.ig >= len(self.d[0]):
+        self.ignored += bytes
+        self.len -= bytes
+        while self.d and self.ignored >= len(self.d[0]):
             s = self.d.popleft()
-            self.ig -= len(s)
+            self.ignored -= len(s)
         if self.len < 0:
             self.len = 0
         if not self.d:
-            self.ig = 0
+            self.ignored = 0
         assert self._assert_invariants()
 
     def copy(self):
         n = self.__class__()
-        n.ig = self.ig
+        n.ignored = self.ignored
         n.len = self.len
         n.d = copy.copy(self.d)
         return n
 
     def _assert_invariants(self):
-        _assert(self.ig >= 0, self.ig)
-        _assert(self.tailig >= 0, self.tailig)
+        _assert(self.ignored >= 0, self.ignored)
+        _assert(self.tailignored >= 0, self.tailignored)
         _assert(self.len >= 0, self.len)
         _assert((not self.d) or (self.d[0]), "First element is required to be non-empty.", self.d and self.d[0])
-        _assert((not self.d) or (self.ig < len(self.d[0])), self.ig, self.d and len(self.d[0]))
-        _assert((not self.d) or (self.tailig < len(self.d[-1])), self.tailig, self.d and len(self.d[-1]))
-        _assert(self.ig+self.len+self.tailig == sum([len(x) for x in self.d]), self.ig, self.len, self.tailig, sum([len(x) for x in self.d]))
+        _assert((not self.d) or (self.ignored < len(self.d[0])), self.ignored, self.d and len(self.d[0]))
+        _assert((not self.d) or (self.tailignored < len(self.d[-1])), self.tailignored, self.d and len(self.d[-1]))
+        _assert(self.ignored+self.len+self.tailignored == sum([len(x) for x in self.d]), self.ignored, self.len, self.tailignored, sum([len(x) for x in self.d]))
         return True
 
     def _collapse(self):
@@ -120,13 +164,13 @@ class StringChain(object):
         of the bytes, so don't do this unless you need to.) """
         assert self._assert_invariants()
         # First trim off any leading ignored bytes.
-        if self.ig:
-            self.d[0] = self.d[0][self.ig:]
-            self.ig = 0
+        if self.ignored:
+            self.d[0] = self.d[0][self.ignored:]
+            self.ignored = 0
         # Then any tail ignored bytes.
-        if self.tailig:
-            self.d[-1] = self.d[-1][:-self.tailig]
-            self.tailig = 0
+        if self.tailignored:
+            self.d[-1] = self.d[-1][:-self.tailignored]
+            self.tailignored = 0
         if len(self.d) > 1:
             newstr = ''.join(self.d)
             self.d.clear()
