@@ -1,10 +1,12 @@
 import unittest
-from stringchain import stringchain
+from stringchain.stringchain import StringChain as BufClass
+# from stringchain.bench.bench import Stringy as BufClass
+# from stringchain.bench.bench import StringIOy as BufClass
+# from stringchain.bench.bench import SimplerStringChain as BufClass
 
-print "stringchain is ", stringchain
 class T(unittest.TestCase):
     def test_al(self):
-        c = stringchain.StringChain()
+        c = BufClass()
         c.append("ab")
         self.failUnlessEqual(len(c), 2)
         c.append("")
@@ -13,17 +15,17 @@ class T(unittest.TestCase):
         self.failUnlessEqual(len(c), 3)
 
     def test_str(self):
-        c = stringchain.StringChain()
+        c = BufClass()
         c.append("ab")
         c.append("c")
         self.failUnlessEqual(str(c), "abc")
 
     def test_trim(self):
-        c = stringchain.StringChain()
+        c = BufClass()
         c.append("ab")
         c.append("c")
         c.trim(1)
-        self.failUnlessEqual(str(c.copy()), "bc", (str(c.copy()), "bc", c.ig, c.len, c.d))
+        self.failUnlessEqual(str(c.copy()), "bc")
         c.trim(1)
         self.failUnlessEqual(str(c.copy()), "c")
         c.trim(1)
@@ -45,54 +47,106 @@ class T(unittest.TestCase):
         c.trim(4) # We just silently trim all.
         self.failUnlessEqual(str(c.copy()), "")
 
-    def test_popleft(self):
-        c = stringchain.StringChain()
+    def test_popleft_new_stringchain(self):
+        c = BufClass()
         c.append("ab")
-        s = c.popleft(1)
+        s = c.popleft_new_stringchain(1)
         self.failUnlessEqual(str(s), "a")
-        self.failUnlessEqual(str(c.copy()), "b", (str(c.copy()), "b", c.ig, c.len, c.d))
-        s = c.popleft(1)
+        self.failUnlessEqual(str(c.copy()), "b")
+        s = c.popleft_new_stringchain(1)
         self.failUnlessEqual(str(s), "b")
         self.failUnlessEqual(str(c.copy()), "")
 
         c.append("abc")
-        s = c.popleft(1)
+        s = c.popleft_new_stringchain(1)
         self.failUnlessEqual(str(s), "a")
         self.failUnlessEqual(str(c.copy()), "bc")
-        s = c.popleft(1)
+        s = c.popleft_new_stringchain(1)
         self.failUnlessEqual(str(s), "b")
         self.failUnlessEqual(str(c.copy()), "c")
-        s = c.popleft(1)
+        s = c.popleft_new_stringchain(1)
         self.failUnlessEqual(str(s), "c")
         self.failUnlessEqual(str(c.copy()), "")
 
         c.append("abc")
-        s = c.popleft(2)
+        s = c.popleft_new_stringchain(2)
         self.failUnlessEqual(str(s), "ab")
         self.failUnlessEqual(str(c.copy()), "c")
-        s = c.popleft(1)
+        s = c.popleft_new_stringchain(1)
         self.failUnlessEqual(str(s), "c")
         self.failUnlessEqual(str(c.copy()), "")
 
         c.append("ab")
         c.append("c")
-        s = c.popleft(2)
+        s = c.popleft_new_stringchain(2)
         self.failUnlessEqual(str(s), "ab")
         self.failUnlessEqual(str(c.copy()), "c")
-        s = c.popleft(1)
+        s = c.popleft_new_stringchain(1)
         self.failUnlessEqual(str(s), "c")
         self.failUnlessEqual(str(c.copy()), "")
 
         c.append("a")
         c.append("bc")
-        s = c.popleft(2)
+        s = c.popleft_new_stringchain(2)
         self.failUnlessEqual(str(s), "ab")
         self.failUnlessEqual(str(c.copy()), "c")
-        s = c.popleft(1)
+        s = c.popleft_new_stringchain(1)
         self.failUnlessEqual(str(s), "c")
         self.failUnlessEqual(str(c.copy()), "")
 
         c.append("abc")
-        s = c.popleft(4) # We just silently pop them all.
+        s = c.popleft_new_stringchain(4) # We just silently pop them all.
         self.failUnlessEqual(str(s), "abc")
         self.failUnlessEqual(str(c.copy()), "")
+
+    def test_popleft(self):
+        c = BufClass()
+        c.append("ab")
+        s = c.popleft(1)
+        self.failUnlessEqual(s, "a")
+        self.failUnlessEqual(str(c.copy()), "b")
+        s = c.popleft(1)
+        self.failUnlessEqual(s, "b")
+        self.failUnlessEqual(str(c.copy()), "")
+
+        c.append("abc")
+        s = c.popleft(1)
+        self.failUnlessEqual(s, "a")
+        self.failUnlessEqual(str(c.copy()), "bc")
+        s = c.popleft(1)
+        self.failUnlessEqual(s, "b")
+        self.failUnlessEqual(str(c.copy()), "c")
+        s = c.popleft(1)
+        self.failUnlessEqual(s, "c")
+        self.failUnlessEqual(str(c.copy()), "")
+
+        c.append("abc")
+        s = c.popleft(2)
+        self.failUnlessEqual(s, "ab")
+        self.failUnlessEqual(str(c.copy()), "c")
+        s = c.popleft(1)
+        self.failUnlessEqual(s, "c")
+        self.failUnlessEqual(str(c.copy()), "")
+
+        c.append("ab")
+        c.append("c")
+        s = c.popleft(2)
+        self.failUnlessEqual(s, "ab")
+        self.failUnlessEqual(str(c.copy()), "c")
+        s = c.popleft(1)
+        self.failUnlessEqual(s, "c")
+        self.failUnlessEqual(str(c.copy()), "")
+
+        c.append("a")
+        c.append("bc")
+        s = c.popleft(2)
+        self.failUnlessEqual(s, "ab")
+        self.failUnlessEqual(str(c.copy()), "c")
+        s = c.popleft(1)
+        self.failUnlessEqual(s, "c")
+        self.failUnlessEqual(str(c.copy()), "")
+
+        c.append("abc")
+        s = c.popleft(4) # We just silently pop them all.
+        self.failUnlessEqual(s, "abc")
+        self.failUnlessEqual(str(c.copy()), "")
